// Extension Preferences Menu
// Author: Antonius32
// Version 2.13.9 (February 2016)

var ExtensionPreferencesMenu = {
	startup: function() {
		window.removeEventListener("load", function EPMload() { ExtensionPreferencesMenu.startup() });

		Components.utils.import("resource://gre/modules/AddonManager.jsm");
		AddonManager.addAddonListener(this);

		Components.utils.import("resource://gre/modules/Services.jsm");
		this._branch = Services.prefs.getBranch("extensions.ExtensionPreferencesMenu.");
		this._branch.addObserver("", this, false);

		// Transfer the values of preferences that have been renamed,
		// then delete the old preferences
		if (this._branch.prefHasUserValue("IconInToolsMenu")) {
			let EPMoldValue = this._branch.getBoolPref("IconInToolsMenu");
			this._branch.setBoolPref("ToolsMenu_ShowIcon", EPMoldValue);
			this._branch.clearUserPref("IconInToolsMenu");
		}
		if (this._branch.prefHasUserValue("BlackList")) {
			let EPMoldValue = this._branch.getCharPref("BlackList");
			this._branch.setCharPref("Blacklist", EPMoldValue);
			this._branch.clearUserPref("BlackList");
		}

		window.addEventListener("customizationchange", function EPMcustomizationchange() { ExtensionPreferencesMenu.setup() });

		if (this._branch.getBoolPref("FirstRun"))
			this.onFirstRun();

		this.setup();
	},

	observe: function(aSubject, aTopic, aData) {
		if (aTopic == "nsPref:changed")
			this.setup();
	},

	onFirstRun: function() {
		this._branch.setBoolPref("FirstRun", false);

		// Pale Moon
		let navBar = document.getElementById("nav-bar");
		if (navBar) {
			// Add the toolbar button to the Navigation toolbar
			let downloadsButton = document.getElementById("downloads-button");
			navBar.contains(downloadsButton) ? navBar.insertItem("extensionPrefsMenu-button", downloadsButton.nextSibling) : navBar.insertItem("extensionPrefsMenu-button");
			navBar.setAttribute("currentset", navBar.currentSet);
			document.persist("nav-bar", "currentset");

			// Make sure the Navigation toolbar is visible
			if (navBar.collapsed) {
				navBar.collapsed = false;
				document.persist("nav-bar", "collapsed");
			}
		}

		// FossaMail
		let mailBar = document.getElementById("mail-bar3");
		if (mailBar) {
			// Add the toolbar button to the Mail toolbar
			let globalSearchBox = document.getElementById("gloda-search");
			mailBar.contains(globalSearchBox) ? mailBar.insertItem("extensionPrefsMenu-button", globalSearchBox.nextSibling) : mailBar.insertItem("extensionPrefsMenu-button");
			mailBar.setAttribute("currentset", mailBar.currentSet);
			document.persist("mail-bar3", "currentset");

			// Make sure the Mail toolbar is visible
			if (mailBar.collapsed) {
				mailBar.collapsed = false;
				document.persist("mail-bar3", "collapsed");
			}
		}
	},

	setup: function() {
		let EPMappMenu = document.getElementById("appmenu_extensionPrefs");
		EPMappMenu.collapsed = !this._branch.getBoolPref("InAppMenu");
		let origAppMenu = document.getElementById("appmenu_addons");
		origAppMenu.collapsed = !EPMappMenu.collapsed;

		this.populate(EPMappMenu.firstChild);

		let EPMtoolsMenu1 = document.getElementById("extensionPrefsMenu1");
		EPMtoolsMenu1.collapsed = !(this._branch.getBoolPref("InToolsMenu") && this._branch.getBoolPref("ToolsMenu_ReplaceOriginal"));
		let EPMtoolsMenu2 = document.getElementById("extensionPrefsMenu2");
		EPMtoolsMenu2.collapsed = !(this._branch.getBoolPref("InToolsMenu") && !this._branch.getBoolPref("ToolsMenu_ReplaceOriginal"));
		let origToolsMenu = document.getElementById("menu_openAddons") || document.getElementById("addonsManager");
		origToolsMenu.collapsed = !EPMtoolsMenu1.collapsed;

		this._branch.getBoolPref("ToolsMenu_ShowIcon") ? EPMtoolsMenu1.setAttribute("iconic", "true") : EPMtoolsMenu1.removeAttribute("iconic");
		this._branch.getBoolPref("ToolsMenu_ShowIcon") ? EPMtoolsMenu2.setAttribute("class", "menu-iconic") : EPMtoolsMenu2.setAttribute("class", "menu");

		this.populate(EPMtoolsMenu1.firstChild);
		this.populate(EPMtoolsMenu2.firstChild);

		let EPMtoolbarButton = document.getElementById("extensionPrefsMenu-button");
		if (EPMtoolbarButton)
			this.populate(EPMtoolbarButton.firstChild);
	},

	populate: function(aParent) {
		let item, MenuIconURL, EPMshowIt = true, EPMblacklistArray = [];
		let EPMshowVersionNumbers = this._branch.getBoolPref("ShowVersionNumbers");
		let EPMshowIcons = this._branch.getBoolPref("ShowIcons");
		let EPMshowAll = this._branch.getBoolPref("ShowAll");
		let EPMshowDisabled = this._branch.getBoolPref("ShowDisabled");
		let EPMshowIncompatible = this._branch.getBoolPref("ShowIncompatible");
		let EPMquickActions = this._branch.getBoolPref("QuickActions");
		let EPMshowRestart = this._branch.getBoolPref("ShowRestart");
		let EPMrestartLabel = document.getElementById("EPMStringBundle").getString("EPMRestart");
		let EPMopenApp = window["BrowserOpenAddonsMgr"] || window["openAddonsMgr"]; 
		let EPMscrollInToView = this._branch.getBoolPref("ScrollInToView");
		let EPMhideSigned = this._branch.getBoolPref("HideSigned");
		let EPMblacklist = this._branch.getCharPref("Blacklist");
		if (EPMblacklist) {
			EPMblacklist = EPMblacklist.split("").reverse().join("");
			EPMblacklistArray = EPMblacklist.split(/\,(?!\\)/);
			for (let i = 0; i < EPMblacklistArray.length; i++)
				EPMblacklistArray[i] = EPMblacklistArray[i].split("").reverse().join("").replace(/^ | $/g,"").replace(/\\\,/g,",").toLowerCase();
		}

		AddonManager.getAllAddons(function(addon) {
			while (aParent.childNodes.length)
				aParent.removeChild(aParent.firstChild);

			addon.forEach(function(EPMaddon) {
				if (EPMaddon.type == "extension" && (!EPMaddon.appDisabled || (EPMaddon.appDisabled && EPMshowIncompatible)) && ((EPMaddon.isActive && EPMaddon.optionsURL) || ((EPMaddon.userDisabled && EPMshowDisabled) || (!EPMaddon.userDisabled && EPMshowAll) || (EPMaddon.appDisabled && EPMshowIncompatible)))) {
					item = document.createElement("menuitem");
					item.setAttribute("closemenu", "none");
					item.setAttribute("label", EPMshowVersionNumbers ? EPMaddon.name + "  " + (EPMhideSigned ? EPMaddon.version.replace(".1-signed", ".1") : EPMaddon.version) : EPMaddon.name);
					if (EPMaddon.description)
						item.setAttribute("tooltiptext", EPMaddon.description);
					item.addEventListener("command", function(event) {
						if (EPMaddon.optionsURL && !event.ctrlKey) {
							ExtensionPreferencesMenu.closeMenu();
							switch (Number(EPMaddon.optionsType)) {
								case 2:
									EPMopenApp("addons://detail/" + encodeURIComponent(EPMaddon.id) + (EPMscrollInToView ? "/preferences" : ""));
									break;
								case 3:
									switchToTabHavingURI(EPMaddon.optionsURL, true);
									break;
								default:
									var windows = Services.wm.getEnumerator(null);
									while (windows.hasMoreElements()) {
										var win = windows.getNext();
										if (win.document.documentURI == EPMaddon.optionsURL) {
											win.focus();
											return;
										}
									}
									let instantApply = Application.prefs.get("browser.preferences.instantApply");
									let features = "chrome,titlebar,toolbar,centerscreen" + (instantApply.value ? ",dialog=no" : ",modal");
									openDialog(EPMaddon.optionsURL, EPMaddon.name, features);
							}
						}
					});
					item.addEventListener("click", function(event) {
						if (EPMquickActions) {
							if (event.button == 1 && EPMaddon.homepageURL) {
								if ("gBrowser" in window) { // Pale Moon
									ExtensionPreferencesMenu.closeMenu();
									switchToTabHavingURI(EPMaddon.homepageURL, true);
								}
								else // FossaMail
									Cc["@mozilla.org/messenger;1"].createInstance(Ci.nsIMessenger).launchExternalURL(EPMaddon.homepageURL);
							}
							if (event.button == 2 || (event.button == 0 && event.ctrlKey))
								AddonManager.getAddonByID(EPMaddon.id, function(addon) {
									addon.userDisabled = !addon.userDisabled;
									if (addon.operationsRequiringRestart && !EPMshowRestart)
										Application.restart();
									ExtensionPreferencesMenu.setup();
								});
						}
					});
					if (!EPMaddon.optionsURL)
						item.setAttribute("disabled", true);
					if (!EPMaddon.isActive)
						item.setAttribute("EPMisDisabled", true);
					if (EPMaddon.pendingOperations == 1)
						item.setAttribute("EPMtoBeToggled", true);
					if (EPMaddon.pendingOperations == 2)
						item.setAttribute("EPMtoBeToggled", false);

					if (EPMshowIcons) {
						item.setAttribute("class", "menuitem-iconic");
						MenuIconURL = EPMaddon.iconURL || "chrome://mozapps/skin/extensions/extensionGeneric-16.png";
						item.setAttribute("style", "list-style-image: url(" + MenuIconURL + ")");
					}
					else
						item.setAttribute("class", "menuitem");

					if (EPMblacklistArray)
						for (let i = 0; i < EPMblacklistArray.length; i++)
							if (EPMblacklistArray[i] == EPMaddon.name.toLowerCase()) {
								EPMshowIt = false;
								break;
							}
							else
								EPMshowIt = true;

					if (EPMshowIt)
						aParent.appendChild(item);
				}
			});

			Cc["@mozilla.org/xul/xul-sort-service;1"].getService(Ci.nsIXULSortService).sort(aParent, "label", "ascending");

			if (EPMshowRestart) {
				aParent.appendChild(document.createElement("menuseparator"));

				item = document.createElement("menuitem");
				item.setAttribute("label", EPMrestartLabel);
				item.addEventListener("command", function() { Application.restart() });

				if (EPMshowIcons) {
					item.setAttribute("class", "menuitem-iconic");
					item.setAttribute("style", "list-style-image: url(chrome://prefsmenu/skin/restart.png)");
				}
				else
					item.setAttribute("class", "menuitem");

				aParent.appendChild(item);
			}
		});
	},

	closeMenu: function() {
		let appMenuPopup = document.getElementById("appmenu-popup");
		if (appMenuPopup.state == "open") {
			appMenuPopup.hidePopup();
			return;
		}
		let toolsMenuPopup = document.getElementById("menu_ToolsPopup") || document.getElementById("taskPopup");
		if (toolsMenuPopup.state == "open") {
			toolsMenuPopup.hidePopup();
			return;
		}
		let EPMbuttonPopup = document.getElementById("EPMButtonPopup");
		EPMbuttonPopup.hidePopup();
	},

	openAddonsManager: function() {
		(window["BrowserOpenAddonsMgr"] || window["openAddonsMgr"])((this._branch.getBoolPref("OpenExtensionsPage")) ? "addons://list/extension" : "");
	},

	shutdown: function() {
		window.removeEventListener("unload", function EPMunload() { ExtensionPreferencesMenu.shutdown() });
		AddonManager.removeAddonListener(this);
		this._branch.removeObserver("", this);
		window.removeEventListener("customizationchange", function EPMcustomizationchange() { ExtensionPreferencesMenu.setup() });
	}
};

window.addEventListener("load", function EPMload() { ExtensionPreferencesMenu.startup() });
window.addEventListener("unload", function EPMunload() { ExtensionPreferencesMenu.shutdown() });
